﻿using Bogus;
using FluentAssertions;
using Microsoft.EntityFrameworkCore;
using NSubstitute;
using System;
using System.Collections.Generic;
using System.Linq.Expressions;
using System.Threading.Tasks;
using VV.Wrappers.SqlServer.Core.EF.Repositories;
using VV.Wrappers.SqlServer.Core.EF.Repositories.Interfaces;
using VV.Wrappers.SqlServer.Core.Tests.Comum.Context;
using VV.Wrappers.SqlServer.Core.Tests.Comum.Entities;
using VV.Wrappers.SqlServer.Core.Tests.Comum.Faker;
using Xunit;

namespace VV.Wrappers.SqlServer.Core.Tests.Repositories
{
    public class RepositoryEFBaseTest
    {
        private const int Received = 1;

        private IEFContext efContext;
        private RepositoryEFBase<PersonTest, int> repositoryBase;
        private Faker faker;

        public RepositoryEFBaseTest()
        {
            faker = new Faker();
            efContext = Substitute.For<IEFContext>();
            
            var dbOptions = new DbContextOptionsBuilder<PersonTextContext>()
                                .UseInMemoryDatabase(databaseName: "ToDoDb")
                                .Options;

            var context = new PersonTextContext(dbOptions);
                efContext.Context().Returns(context);

            repositoryBase = Substitute.For<RepositoryEFBase<PersonTest, int>>(efContext);
        }

        [Fact]
        public async Task ShouldAbstractInsertAsyncWithSuccess()
        {
            //arrange
            var person = GeneratePersonFaker.CreatePersonObjectFaker();
            repositoryBase.When(x => x.InsertAsync(Arg.Any<PersonTest>())).CallBase();

            //act
            await repositoryBase
                    .InsertAsync(person)
                    .ConfigureAwait(false);

            //assert
            await repositoryBase
                    .Received(Received)
                    .InsertAsync(Arg.Any<PersonTest>())
                    .ConfigureAwait(false);
        }

        [Fact]
        public async Task ShouldAbstractUpdateAsyncWithSuccess()
        {
            //arrange
            var person = GeneratePersonFaker.CreatePersonObjectFaker();
            repositoryBase.When(x => x.UpdateAsync(Arg.Any<PersonTest>())).CallBase();
            
            repositoryBase
                .GetByIdAsync(person.Id)
                .Returns(person);

            //act
            await repositoryBase
                    .UpdateAsync(person)
                    .ConfigureAwait(false);

            //assert
            await repositoryBase
                    .Received(Received)
                    .UpdateAsync(Arg.Any<PersonTest>())
                    .ConfigureAwait(false);

            await repositoryBase
                    .Received(Received)
                    .GetByIdAsync(Arg.Any<int>())
                    .ConfigureAwait(false);
        }

        [Fact]
        public async Task ShouldAbstractGetByIdAsyncWithSuccess()
        {
            //arrange
            var person = GeneratePersonFaker.CreatePersonObjectFaker();

            repositoryBase
                .When(x => x.GetByIdAsync(Arg.Any<int>()))
                .CallBase();

            repositoryBase
                .GetByIdAsync(person.Id)
                .Returns(person);

            //act
            var preview = await repositoryBase
                                    .GetByIdAsync(person.Id)
                                    .ConfigureAwait(false);

            //assert
            preview
                .Should()
                .BeOfType<PersonTest>();

            preview.Id.Should().Equals(person.Id);
            preview.Name.Should().Equals(person.Name);
            preview.Surname.Should().Equals(person.Surname);

            await repositoryBase
                    .Received(Received)
                    .GetByIdAsync(Arg.Any<int>())
                    .ConfigureAwait(false);
        }

        [Fact]
        public async Task ShouldAbstractGetAllAsyncWithSuccess()
        {
            //arrange
            var person = GeneratePersonFaker.CreateListPersonObjectFaker(10);

            repositoryBase
                .When(x => x.GetAllAsync(Arg.Any<Expression<Func<PersonTest, bool>>>()))
                .CallBase();

            repositoryBase
                .GetAllAsync(a => a.Name == "test")
                .Returns(Task.FromResult(person));

            //act
            var preview = await repositoryBase
                                    .GetAllAsync(a => a.Name == "test")
                                    .ConfigureAwait(false);

            //assert
            await repositoryBase
                    .Received(Received)
                    .GetAllAsync(Arg.Any<Expression<Func<PersonTest, bool>>>())
                    .ConfigureAwait(false);
        }
    }
}
