﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using System.Threading.Tasks;

using VV.Wrappers.SqlServer.Core.EF.Repositories.Interfaces;
using VV.Wrappers.SqlServer.Core.Base.Entities;

using Microsoft.EntityFrameworkCore;

namespace VV.Wrappers.SqlServer.Core.EF.Repositories
{
    public abstract class RepositoryEFBase<TEntity, TKey> : IRepositoryEFBase<TEntity, TKey>, IDisposable
            where TEntity : EntityBase<TKey>
            where TKey : struct
    {
        protected readonly DbSet<TEntity> _db;
        protected readonly DbContext _context;

        protected RepositoryEFBase(IEFContext efContext)
        {
            _context = efContext.Context();
            _db = _context.Set<TEntity>();
        }

        public virtual async Task<TEntity> UpdateAsync(TEntity entity)
        {
            var currentEntity = await GetByIdAsync(entity.Id)
                                        .ConfigureAwait(false);

            _context.Entry(currentEntity).CurrentValues.SetValues(entity);
            
            await SaveChangesAsync()
                    .ConfigureAwait(false);

            return entity;
        }

        public virtual async Task DeleteAsync(TEntity entity)
        {
            _db.Remove(entity);
            await SaveChangesAsync()
                    .ConfigureAwait(false);
        }

        public virtual async Task InsertAsync(TEntity entity)
        {
            await _db
                    .AddAsync(entity)
                    .ConfigureAwait(false);

            await SaveChangesAsync()
                    .ConfigureAwait(false);
        }

        public virtual async Task<IEnumerable<TEntity>> GetAllAsync(Expression<Func<TEntity, bool>> where)
        {
            var query =
                 await _db
                        .Where(where)
                        .OrderByDescending(ent => ent.Id)
                        .AsNoTracking()
                        .ToListAsync()
                        .ConfigureAwait(false);

            return query;
        }

        public virtual async Task<TEntity> GetByIdAsync(TKey id)
        {
            Expression<Func<TEntity, bool>> predicate = t => t.Id.ToString() == id.ToString();
            
            return await _context
                            .Set<TEntity>()
                            .AsNoTracking()
                            .FirstOrDefaultAsync(predicate)
                            .ConfigureAwait(false);
        }

        private async Task SaveChangesAsync()
        {
            if (_context.Database.CurrentTransaction == null)
            {
                await _context
                        .SaveChangesAsync()
                        .ConfigureAwait(false);
            }
        }

        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        protected virtual void Dispose(bool disposing)
        {
            _context.Dispose();
        }
    }
}
