# Utilização do SQL Server em aplicações em .Net Core

Adapter para conexão com o SQL Server  


## Arquivo de configuração - String de conexão

Primeiro passo é adicionar a string de conexão de seu banco de dados SQL Server em seu arquivo de configuração **appsettings.json**

```
"Connection": "Data Source=localhost;Initial Catalog=TesteDb;User Id=sa;Password=123Aa321;"
```

## ORM´s
Neste wrapper contemplam o uso de dois ORM´s que são o Dapper e o EnityFrameworkCore. Você pode utilizar qualquer um dos dois para se conectar em seu banco de dados. A seguir mostrarei o uso de um e de outro.

## Dapper - Injetar o uso do Dapper em sua aplicação

No arquivo Startup.cs de sua aplicação adicione no método **ConfigureServices** o middleware em específico:
```
        public void ConfigureServices(IServiceCollection services)
        {
            ..
            ..
            services.AddDapperSqlServer(Configuration.GetSection("Connection").Value);
        }
```
## Herança na classe de domínio

Nas classes de domínio que serão suas tabelas no SQL Server, realizar a herança da classe **Entity < PrimaryKey >** 
Onde **PrimaryKey**, é o tipo de dado que será a identificação do objeto no SQL Server.

```
public class Person : Entity<int>
    {
        public Person(string name, string surname)
        {
            Name = name;
            Surname = surname;
        }


        public string Name { get; private set; }
        public string Surname { get; private set; }
    }
```  
## Herança de Interface de Repositório Base
Na interface de repositório de sua classe de domínio, precisaremos da herança da interface **IRepositoryDapperBase<ClassDomain, PrimaryKey>**
Onde **ClassDomain** informe sua classe de domínio
Onde **PrimaryKey** informe o tipo da identificação do objeto no SQL Server

````
    public interface IPersonRepository : IRepositoryDapperBase<Person, int>
    {
    }
````

## Herança de Repositório Base
Em seu repositório use a herança da classe **RepositoryDapperBase<ClassDomain, PrimaryKey>**.
Onde **ClassDomain** informe sua classe de domínio
Onde **PrimaryKey** informe o tipo da identificação do objeto no SQL Server

````
public class PersonRepository : RepositoryDapperBase<Person, int>, IPersonRepository
{
   public PersonRepository(IDbConnection connection,
                            ITransactionBase transactionBase) 
     : base (connection, transactionBase)
   {
           
   }
}
````
A interface **ITransactionBase**, é responsavel pelo controle de transação, caso você utilize transação em seus comandos. Veremos como fazer isso no tópico **Transação**.

Com essa herança, você terá os métodos:
- InsertAsync(ClassDomain)
- UpdateAsync(ClassDomain)
- DeleteAsync(PrimaryKey)
- GetByIdAsync(PrimaryKey)
- GetAllAsync()

## Sobrescrita dos métodos base

Caso necessite você poderá fazer a sobrescrita dos métodos da classe base **RepositoryDapperBase<ClassDomain, PrimaryKey>**
```
    public class PersonRepository : RepositoryDapperBase<Person, int>, IPersonRepository
    {
        public PersonRepository(IDbConnection connection,
                                ITransactionBase transactionBase)
            : base(connection, transactionBase)
        {

        }

        public override Task InsertAsync(Person entity)
        {
            return _connection
                    .ExecuteAsync("INSERT INTO PERSON VALUES (@name, @surname)", entity);
        }

        public override Task<Person> GetByIdAsync(int id)
        {
            return base.GetByIdAsync(id);
        }
    }
```
Perceba, que para obter a conexão com o SQL SERVER, existe a propriedade **_connection**, que já está injetada com a sua classe de domínio.

## Transação com Dapper
Caso você precise controlar transação em seu projeto, segue um exemplo da utilização com o design de Unit of Work.
Faça a inversão de controle da interface **IDapperUnitOfWork**, que já está injetada em seu projeto conforme tópico **Dapper - Injetar o uso do Dapper em sua aplicação**

````
public class PersonAppService : IPersonAppService
{
    readonly IPersonRepository _personRepository;
    readonly IDapperUnitOfWork _unitOfWork;

    public PersonAppService(IPersonRepository personRepository,
                            IDapperUnitOfWork unitOfWork)
    {
        _personRepository = personRepository;
        _unitOfWork = unitOfWork;
    }

    public async Task InsertAsync()
    {
        using (transaction = _unitofWork.BeginTransaction())
        {
            try
            {
                await _personRepository
                        .InsertAsync(new Person("Fernando", "Abreu Mendes"))
                        .ConfigureAwait(false);

                transaction.Commit();
            }
            catch(Exception ex)
            {
                transaction.Rollback();
            }
        }
    }
}
````
## EntityFramework - Injetar o uso do EntityFramework em sua aplicação
No arquivo Startup.cs de sua aplicação adicione as linhas no método **ConfigureServices**:
```
        public void ConfigureServices(IServiceCollection services)
        {
            ..
            ..
            services.AddDbContext<PersonContext>(options => options.UseSqlServer(Configuration.GetSection("Connection").Value));
            services.AddScoped<IEFContext>(provider => provider.GetService<PersonContext>());
            services.AddScoped<IEFUnitOfWork, EFUnitOfWork>();
        }
```
## Criação do contexto DbContext
Para que seus domínios estejam disponíveis e injetados no contexto do EntityFramework, precisamos configurá-lo.
Realizar a herança do DbContext em sua classe de contexto, juntamente com a interface **IEFContext**

```
    public class PersonContext : DbContext, IEFContext
    {
        public PersonContext(
            DbContextOptions<PersonContext> options
        ) : base(options)
        {

        }

        public DbSet<Person> Person { get; set; }

        public DbContext Context()
        {
            return this;
        }
    }
```

Perceba-se, que a interface **IEFContext** implementa o método **Context()**. Pois com isso, o contexto do EntityFramework será injetado ao controle do wrapper.

## Herança na classe de domínio
Nas classes de domínio que serão suas tabelas no SQL Server, realizar a herança da classe **Entity < PrimaryKey >** 
Onde **PrimaryKey**, é o tipo de dado que será a identificação do objeto no SQL Server.

```
public class Person : Entity<int>
    {
        public Person(string name, string surname)
        {
            Name = name;
            Surname = surname;
        }


        public string Name { get; private set; }
        public string Surname { get; private set; }
    }
```  
## Herança de Interface de Repositório Base
Na interface de repositório de sua classe de domínio, precisaremos da herança da interface **IRepositoryEFBase<ClassDomain, PrimaryKey>**
Onde **ClassDomain** informe sua classe de domínio
Onde **PrimaryKey** informe o tipo da identificação do objeto no SQL Server

````
    public interface IPersonRepository : IRepositoryEFBase<Person, int>
    {
    }
````

## Herança de Repositório Base
Em seu repositório use a herança da classe **RepositoryEFBase<ClassDomain, PrimaryKey>**.
Onde **ClassDomain** informe sua classe de domínio
Onde **PrimaryKey** informe o tipo da identificação do objeto no SQL Server

````
public class PersonRepository : RepositoryEFBase<Person, int>, IPersonRepository
{
   public PersonRepository(IEFContext context) 
     : base (context)
   {
           
   }
}
````
Com essa herança, você terá os métodos:
- InsertAsync(ClassDomain)
- UpdateAsync(ClassDomain)
- DeleteAsync(PrimaryKey)
- GetByIdAsync(PrimaryKey)
- GetAllAsync(Expression)

## Sobrescrita dos métodos base

Caso necessite você poderá fazer a sobrescrita dos métodos da classe base **RepositoryDapperBase<ClassDomain, PrimaryKey>**
```
    public class PersonRepository : RepositoryEFBase<Person, int>, IPersonRepository
    {
        public PersonRepository(IEFContext context)
            : base(context)
        {

        }

        public override async Task InsertAsync(Person entity)
        {
            await _context
                    .AddAsync(entity);
        }
    }
```
Perceba, que para obter a conexão com o SQL SERVER, existe a propriedade **_context**, que já está injetada com a sua classe de domínio.

## Transação com EntityFramework
Caso você precise controlar transação em seu projeto, segue um exemplo da utilização com o design de Unit of Work.
Faça a inversão de controle da interface **IEFUnitOfWork**, que já está injetada em seu projeto conforme tópico **EntityFramework - Injetar o uso do EntityFramework em sua aplicação**

````
public class PersonAppService : IPersonAppService
{
    readonly IPersonRepository _personRepository;
    readonly IEFUnitOfWork _unitOfWork;

    public PersonAppService(IPersonRepository personRepository,
                            IEFUnitOfWork unitOfWork)
    {
        _personRepository = personRepository;
        _unitOfWork = unitOfWork;
    }

    public async Task InsertAsync()
    {
        using (var transaction = await _unitOfWork.BeginTransactionAsync())
        {
            try
            {
                await _personRepository
                        .InsertAsync(new Person("Fernando", "Abreu Mendes"))
                        .ConfigureAwait(false);

                await transaction
                        .CommitAsync()
                        .ConfigureAwait(false);
            }
            catch (Exception ex)
            {
                await transaction
                        .RollbackAsync()
                        .ConfigureAwait(false);
            }
        }
    }
}
````

## Como Publicar no Nexus
```
dotnet nuget push ./VV.Wrappers.SqlServer.Core.1.0.0.nupkg -k "Obtenha a APi Key nos canais de suporte" -s http://nexus.viavarejo.com.br/repository/nuget.org-proxy/ 
```

## Como adicionar no ambiente local o repository/nuget utilizando o ''dotnet''
```
dotnet nuget add source http://nexus.viavarejo.com.br/repository/nuget.org-proxy/index.jso
```
